package ua.in.hutorny.fsa.primer;

import ua.in.hutorny.fsa.model.Action;
import ua.in.hutorny.fsa.model.Condition;
import ua.in.hutorny.fsa.model.Event;
import ua.in.hutorny.fsa.model.State;
import ua.in.hutorny.fsa.util.*;

public class MicrowaveSample implements WalkerCallback {
	
	public static void main(String args[]) {
		Walker walker = new Walker(new MicrowaveSample());
		out(";-----------------------------------------------------");
		walker.processAutomaton(MicrowaveStateMachine.class);
		out(";-----------------------------------------------------");
		walker.processAutomaton(ModernMicrowaveStateMachine.class);
	}

	private String labelNextTransition = "";
	private int count = 0;
	private String onLeaveActions = "";

	public void stateBegin(Class<? extends State> state) {
		System.out.println(state.getSimpleName());
	}

	public void actionsBegin(Class<? extends State> state) {
		onLeaveActions = "";
	}

	public void actionOnEnter(Class<? extends State> state, String name,
			Class<? extends Action> action) {
		out("    call " + action.getSimpleName());
	}

	public void actionOnLeave(Class<? extends State> state, String name,
			Class<? extends Action> action) {
		if( onLeaveActions.length() > 0 )
			onLeaveActions += "\r\n";
		onLeaveActions += "    call " + action.getSimpleName();
	}

	public void actionsEnd(Class<? extends State> state) {
		System.out.println(state.getSimpleName() + "_begin");
		System.out.println("    call sleepAndWaitEvents");
	}

	public void transitonsBegin(Class<? extends State> state) {
		count = 0;
		labelNextTransition = state.getSimpleName() + "_" + count++;
	}

	public void transitionBegin(Class<? extends State> fromState,
			Class<? extends State> toState, Class<? extends Event> event,
			String name) {
		out(labelNextTransition);
		labelNextTransition = fromState.getSimpleName() + "_" + count++;
		out("    btfss " + event.getSimpleName() + "    ; " + name);
		out("    bra " + labelNextTransition );
	}

	public void condition(Class<? extends State> fromState,
			Class<? extends State> toState, Class<? extends Event> event,
			String name, Class<? extends Condition> condition) {
		out("    btfss " + condition.getSimpleName());
		out("    bra " + labelNextTransition);
	}

	public void action(Class<? extends State> fromState,
			Class<? extends State> toState, Class<? extends Event> event,
			String name, Class<? extends Action> action) {
		out("    call " + action.getSimpleName());
	}

	public void transitionEnd(Class<? extends State> fromState,
			Class<? extends State> toState, Class<? extends Event> event,
			String name) {
		if( onLeaveActions.length() > 0 ) {
			out("    call " + fromState.getSimpleName() + "_leave" );
		}
		out("    bra " + toState.getSimpleName());
	}

	public void transitonsEnd(Class<? extends State> state) {
		out(labelNextTransition);
	}

	public void stateEnd(Class<? extends State> state) {
		out(state.getSimpleName()+ "_end");
		out("    bra " + state.getSimpleName()+ "_begin");
		if( onLeaveActions.length() > 0 ) {
			out(state.getSimpleName()+ "_leave");
			out(onLeaveActions);
			out("    return");
		}
	}
	
	protected static void out(String string) {
		System.out.println(string);
	}
}