package ua.in.hutorny.fsa.util;

import ua.in.hutorny.fsa.model.*;

/**
 * Walker Callback interface methods are called by the walker 
 * on certain phases of FSA processing
 * Methods are given in the expected order   
 * @author Eugene M. Hutorny &lt;eugene@hutorny.in.ua&gt;
 *
 */
public interface WalkerCallback {
	/**
	 * Called when Walker begins processing an automaton state
	 * @param state
	 */
	public void stateBegin(Class<? extends State> state);

	/**
	 * Called before processing OnEnter and onLeave actions 
	 * @param state
	 */
	public void actionsBegin(Class<? extends State> state);

	/**
	 * Called when Walker has encountered an OnEnter action
 	 * @param state
	 * @param name
	 * @param action
	 */
	public void actionOnEnter(Class<? extends State> state, String name, Class<? extends Action> action);	
	
	/**
	 * Called when Walker has encountered an OnLeave action
	 * @param state
	 * @param name
	 * @param action
	 */
	public void actionOnLeave(Class<? extends State> state, String name, Class<? extends Action> action);	
	
	/**
	 * Called after all OnEnter and OnLeave actions were processed
	 * @param state
	 */
	public void actionsEnd(Class<? extends State> state);
	
	/**
	 * Called before processing transitions 
	 * @param state
	 */
	public void transitonsBegin(Class<? extends State> state);
	
	/**
	 * Called when Walker started processing a transition  
	 * @param fromState
	 * @param toState
	 * @param event
	 * @param name
	 */
	public void transitionBegin(Class<? extends State> fromState, Class<? extends State> toState, Class<? extends Event> event, String name);

	/**
	 * Called when Walker has encountered a condition in the transition
	 * @param fromState
	 * @param toState
	 * @param event
	 * @param name
	 * @param condition
	 */
	public void condition(Class<? extends State> fromState, Class<? extends State> toState, Class<? extends Event> event, String name, Class<? extends Condition> condition);

	/**
	 * Called when Walker has encountered an action in the transition
	 * @param fromState
	 * @param toState
	 * @param event
	 * @param name
	 * @param action
	 */
	public void action(Class<? extends State> fromState, Class<? extends State> toState, Class<? extends Event> event, String name, Class<? extends Action> action);

	/**
	 * Called when Walker finishes processing a transition  
	 * @param fromState
	 * @param toState
	 * @param event
	 * @param name
	 */
	public void transitionEnd(Class<? extends State> fromState, Class<? extends State> toState, Class<? extends Event> event, String name);

	/**
	 * Called when all transitions were processed
	 * @param state
	 */
	public void transitonsEnd(Class<? extends State> state);

	/**
	 * Called when Walker has ended processing a state 
	 * @param state
	 */
	public void stateEnd(Class<? extends State> state);	
}
